// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Test that given rgsoffsetcalc correctly identifes 0/1/both-node readout modes
//
// Giuseppe Vacanti, February 2000
//

#include <Dal.h>
#include <cassert>
#include <iostream>
#include <string>
#include <rgsoffsetcalc.h>
#include <metatask.h>
#include <param.h>
#include <Random.h>
#include <cmath>
#include <unistd.h>

void test_one_node(std::string name, int8 ic, int8 in)
//void test_one_node(std::string name, int8 , int8 )
{
  const std::string oname = "out.ds";
  
  const std::vector<std::string> sets(1,name);
  openParameters("rgsoffsetcalc");
  setParameter("sets", sets);
  setParameter("outset", oname);
  task(rgsoffsetcalc);
  
  DataSet *aset = dataSetServer->open(oname, Dal::Read);
  Table *tab = aset->table("RGSOFFSETS");
  assert(tab->rows() == 1);
  
  // these leaks memory
  ColumnData *ccdid = tab->column("CCD_ID")->data();
  ColumnData *nodeid = tab->column("NODE_ID")->data();
  
  int8 *ccd = ccdid->int8Data();
  int8 *node = nodeid->int8Data();
  
  assert (ccd[0] == ic);
  assert (node[0] == in);
  
  dataSetServer->close(aset);  
  unlink(oname.c_str());
}

int main (){

  std::vector<std::string> sets;

  sets.push_back( "testdata/node0.fits");
  sets.push_back( "testdata/node1.fits");
  

  int8 ics[] = { 2, 2 };
  std::vector<int8> cs(ics, ics+2);

  int8 ins[] = { 0, 1 };
  std::vector<int8> ns(ins, ins+2);

  for(unsigned k = 0 ; k < sets.size(); k++){
    cerr << sets[k] << endl;
    test_one_node(sets[k], ics[k], ins[k]);
  }
}
