// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Test that given a gaussian distribution in an image rgsoffsetcalc 
// can recover the correct mean.
//
// Giuseppe Vacanti, November 1999
//

#include <Dal.h>
#include <cassert>
#include <iostream>
#include <string>
#include <rgsoffsetcalc.h>
#include <metatask.h>
#include <param.h>
#include <Random.h>
#include <cmath>
#include <unistd.h>

std::vector<double> gaussRand(unsigned n) {
  std::vector<double> v;
  bool f = true;
  for(unsigned i=0; i<n; i++){
    double fac = 0.;
    double v1 = 0.;
    double v2 = 0.;
    if(f){
      f = false;
      double r = -1;
      while(r <= 0. || r >= 1.){
	v1 = 2. * randGen.rand() - 1;
	v2 = 2. * randGen.rand() - 1;
	r = v1*v1 + v2*v2;
      }
      fac = std::sqrt(-2.*log(r)/r);
       v.push_back(fac*v1);
    } else {
      f = true;
      v.push_back(fac*v2);
    }
  }
  return v;
}
      

//int main (int argc, char* argv[]){
int main (){

  const std::string name = "abc.ds";
  const std::string oname = "out.ds";
  const unsigned n = 100;
  longVec dim(2,n);

  const double s = 10;
  const double m = 100;

  DataSet *set = dataSetServer->open(name, Dal::Create);
  Array *arr = set->addArray("array", Dal::Int16, dim);
  *arr->addAttribute("BLANK") = 6666;
  const unsigned nel = arr->elements();
  ArrayData *arrd = arr->data();
  int16 *v = arrd->int16Data();
  std::vector<double> g = gaussRand(arr->elements());
  for(unsigned i=0; i<arr->elements(); i++){
    //    std::cout << g[i] << std::endl;
    v[i] = static_cast<int16>(g[i] * s + m);
  }

  dataSetServer->close(set);

  const std::vector<std::string> sets(1,name);
  openParameters("rgsoffsetcalc");
  setParameter("sets", sets);
  setParameter("outset", oname);
  task(rgsoffsetcalc);

  
  DataSet *aset = dataSetServer->open(oname, Dal::Read);
  Table *tab = aset->table("RGSOFFSETS");
  ColumnData *cmean = tab->column("MEAN")->data();
  ColumnData *cstddev = tab->column("STD_DEV")->data();
  real32 *vmean = cmean->real32Data();
  real32 *vstddev = cstddev->real32Data();

  for(unsigned i=0; i<tab->rows(); i++){
    double q = std::fabs((vmean[i] - m)/vstddev[i]/sqrt(static_cast<double>(nel)));
    std::cout << i << " " << q << std::endl;
    assert( q <= 3.);
  }
  delete cmean;
  delete cstddev;
  dataSetServer->close(aset);  
  unlink(name.c_str());
  unlink(oname.c_str());
}

  
  
